import datetime
import json
import os
from concurrent.futures import ThreadPoolExecutor

from typing import Any, Dict, List
from google.cloud import storage
import os
# from dotenv import load_dotenv
# load_dotenv()

class Cloudstore:
    def __init__(self):
        GCP_PROJECT = os.getenv("GCP_PROJECT", "gandai-prod")
        GCP_STAGE = os.getenv("GCP_STAGE", "prod")
        self.BUCKET_NAME = f"{GCP_PROJECT}-{GCP_STAGE}"
        self.client = storage.Client(project=GCP_PROJECT)
        self.bucket = self.client.get_bucket(self.BUCKET_NAME)

    def keys(self, prefix="") -> List[str]:
        keys = [
            blob.name
            for blob in self.client.list_blobs(self.BUCKET_NAME, prefix=prefix)
        ]
        return keys

    def delete(self, key: str) -> None:
        blob = self.bucket.blob(key)
        blob.delete()

    def __getitem__(self, key: str) -> json:
        blob = self.bucket.blob(key)
        return json.loads(blob.download_as_string())

    def __setitem__(self, key: str, data) -> None:
        blob = self.bucket.blob(key)
        blob.upload_from_string(json.dumps(data))

    def load_async(self, keys):
        with ThreadPoolExecutor(max_workers=20) as exec:
            futures = exec.map(self.__getitem__, keys)
        return list(futures)

    def get_signed_url(self, key: str, hours_valid=72) -> str:
        blob = self.bucket.blob(key)
        expiration = datetime.timedelta(hours=hours_valid)
        url = blob.generate_signed_url(
            version="v4", expiration=expiration, method="GET"
        )
        return url



from typing import List, Optional
from enum import Enum, auto
from dataclasses import asdict, dataclass, field

@dataclass
class Actor:
    # id: int = field(init=False)  # pk
    key: str  # phone number
    type: str
    name: str
    # created: int = field(init=False)
    # updated: int = field(init=False)


@dataclass(order=True)
class Company:
    domain: str  # unique
    name: Optional[str] = None
    description: Optional[str] = None
    meta: dict = field(default_factory=dict)
    id: int = field(default=None)  # primary key
    uid: int = field(default=None)  # foreign key


class EventType(str, Enum):
    CREATE = auto()
    ADVANCE = auto()
    VALIDATE = auto()
    SEND = auto()
    CLIENT_ACCEPT = auto()
    CONFLICT = auto()
    REJECT = auto()
    CRITERIA = auto()


@dataclass
class Event:
    
    search_uid: int  # fk # add index 
    domain: Optional[str]  # fk
    actor_key: str  # fk
    type: str  # build, advance, qualify, reject, conflict, rate
    data: dict = field(default_factory=dict)
    id: int = field(default=None)  # pk
    # created: int = field(init=False)


@dataclass
class Comment(Event):
    def __post_init__(self):
        assert isinstance(self.data["comment"], str)


@dataclass
class Rating(Event):
    def __post_init__(self):
        assert isinstance(self.data["rating"], int)

@dataclass
class Checkpoint:
    # actor_key: str  # foreign key
    event_id: int  # foreign key
    id: int  = field(default=None) # primary key
    # created: int  # timestamp

@dataclass(order=True)
class Target:
    domain: str  # enforce unique
    name: str
    search_id: int  # fk
    description: str
    event: List
    last_event_type: str  # enum EventType
    last_rating: int  # enum EventType
    meta: dict


@dataclass
class Criteria:
    search_id: int
    type: str  # include, exclude
    prompt: str
    rating: list
    employees_range: list
    country: list
    state: list
    ownership: list
    product: list
    services: list
    updated: int
    actor_id: int  # fk


@dataclass
class Search:
    
    uid: int  # foreign key, dealcloud id
    client_domain: str  # 
    label: str
    meta: dict = field(default_factory=dict)
    inclusion: dict = field(default_factory=dict)
    exclusion: dict = field(default_factory=dict)
    sort: dict = field(default_factory=dict)
    id: int  = field(default=None) # primary key
    # created: int = field(init=False)
    # updated: int = field(init=False)


@dataclass
class Sort:
    FIELDS = ["employee_count", "rating", "state", "country", "year_founded", "name"]
    ORDERS = ["asc", "desc"]

    field: str
    order: str



from gandai import query
from gandai.models import Search

import re
import pandas as pd


def dealcloud_company_query(
    fp="/Users/parker/Development/gandai-workspace/notebooks/2023-05-13/data/Company_05132023.xlsx",
) -> pd.DataFrame:
    print("dealcloud_company_query")

    def _get_domain(url) -> str:
        url = url.replace("http://", "").replace("https://", "").replace("www.", "")
        return url.split("/")[0]

    df = pd.read_excel(fp)  # hmm why so slow, maybe just select specific columns?
    df.columns = [col.replace(" ", "_").replace(".", "").lower() for col in df.columns]
    assert "dealcloud_id" in df.columns
    df["domain"] = df["website"].dropna().apply(_get_domain)
    df["days_since_contact"] = (
        df["days_since_contact"].dropna().apply(lambda x: int(re.findall(r"\d+", x)[0]))
    )
    df = df.rename(
        columns={"company_name": "name", "business_description": "description"}
    )
    # df = (
    #     df[["dealcloud_id","name", "domain", "days_since_contact"]]
    #     .drop_duplicates(subset=["domain"])
    #     .reset_index(drop=True)
    # )
    return df


def dealcloud_engagements(fp="data/Engagement_05132023.xlsx"):
    df = pd.read_excel(fp)
    df.columns = [col.lower().replace(" ", "_").replace(".", "") for col in df.columns]
    df["dealcloud_id"] = df["dealcloud_id"].astype(str)
    assert "dealcloud_id" in df.columns
    today = pd.to_datetime("today")
    df["modified_days_ago"] = (today - pd.to_datetime(df["modified_date"])).dt.days
    date_cols = [col for col in df.columns if col.endswith("_date")]
    for col in date_cols:
        df[col] = df[col].apply(lambda x: str(x)[0:10])
    df = df.dropna(subset=["engagement_name"])  # never hit this condition
    df = df.sort_values("modified_date", ascending=False)
    df = df[
        ~df["status"].isin(
            ["Lost Pre-Mandate", "Completed Engagement", "Dead Post-Mandate"]
        )
    ].reset_index(drop=True)
    df = df[df["modified_days_ago"] < 365].reset_index(drop=True)
    return df


def migrate_engagements():
    engagements = dealcloud_engagements(
        "/Users/parker/Development/gandai-workspace/notebooks/2023-05-13/data/Engagement_05132023.xlsx"
    )
    companies = dealcloud_company_query(
        "/Users/parker/Development/gandai-workspace/notebooks/2023-05-13/data/Company_05132023.xlsx"
    )

    clients = companies[companies["name"].isin(engagements["client"])]
    clients = clients.rename(columns={"domain": "client_domain"})[
        ["name", "client_domain"]
    ]

    engagements = engagements.merge(
        clients, left_on="client", right_on="name", how="left"
    )

    for engagement in engagements.fillna("").to_dict(orient="records"):
        search = Search(
            uid=int(engagement["dealcloud_id"]),
            client_domain=engagement["client_domain"],
            label=engagement["engagement_name"],
            meta=engagement,
            inclusion={
                "country": ["USA"],
                "employees_range": [10, 100],
            },
            sort={"field": "domain", "order": "desc"},
        )
        print(search)
        query.insert_search(search)


def main():
    migrate_engagements()


if __name__ == "__main__":
    main()
